--- D.lua
-- debug-print for lua values
-- usage:
--   D = require "d"
--   D(<var> [,<depth>])
--
local assert, type, pairs, rep,        sub,        concat       =
      assert, type, pairs, string.rep, string.sub, table.concat 

return setmetatable(
  {
    depth = 2,       -- iteration depth (0 = dont iterate)
    strlen = 80,     -- max. string length to print
    ctrlchrs = true, -- replace control chars
  },
  {
    __call = function(self, var, depth)
      if depth then
        if (depth >= 0) then
          self.depth = depth
        end
      else
        depth = self.depth
      end
      local recursetable = {var}
      local function writeitem(var, level, depth) 
        local t = type(var)
        if t == "nil" then 
          return t
        end
        if t == "string" then
          -- string beautifiing can be done much better than this
          local s = var
          if self.ctrlchrs then
            s = s:gsub("\n", "\\n")
          end
          if self.strlen > 0 then
            s = s:sub(1, self.strlen)
          end
          return('"' .. s .. '"')
        end
        if t == "number" then
          return var
        end
        if t == "boolean" then
          if var then
            return("true")
          else
            return("false")
          end
        end
        if t == "table" and (level <= depth) then 
          local function pairsByKeys(t)
            local a = {}
            for n in pairs(t) do a[#a + 1] = n end
            table.sort(a)
            local i = 0         -- iterator variable
            return function ()  -- iterator function
              i = i + 1
              return a[i], t[a[i]]
            end
          end
          local function indent(i)
            return rep("  ", i)
          end
          local rtptr = #recursetable
          local function isrecursive(v)
            if type(v) ~= "table" then return false; end;
            for i = 1, rtptr do
              if v == recursetable[i] then return i; end;
            end
            return false
          end
          local st = {}
          st[#st+1] = "{ -- #"..isrecursive(var).."\n"
          for _, v in pairsByKeys(var) do
            if (type(v) == "table") and not isrecursive(v) then
              recursetable[#recursetable+1] = v
            end
          end
          for k, v in pairsByKeys(var) do
            st[#st+1] = indent(level + 1)
            if type(k) == "number" then
              st[#st+1] =  "[" .. k .. "] = "
            else
              st[#st+1] =  k .. " = "
            end
            if v == _G then
              st[#st+1] = "_G" 
            else
              local ir = isrecursive(v)
              if ir then
                st[#st+1] = "<Table#".. ir ..">"
              else
                st[#st+1] = writeitem(v, level + 1, depth, recursetable)
              end
            end
            st[#st+1] = ",\n"
          end
          st[#st+1] = indent(level) .. "}"
          return(concat(st))
        end -- table
        return "<"..t..">"
      end
      print(writeitem(var, 0, depth))
    end
  }
)

